

/*
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * <julianacamilo_@hotmail.com> and <thiagums@hotmail.com> wrote this file.
 * As long as you retain this notice you can do whatever you want with this stuff.
 * If we meet some day, and you think this stuff is worth it, you can buy
 * me a beer in return Juliana Camilo Inácio e Thiago Martins de Sousa
 * ----------------------------------------------------------------------------
 */

/*
 * Tradução da Licensa
 *  ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * <julianacamilo_@hotmail.com> e <thiagums@hotmail.com>  escreveram esse código.
 * Enquanto você possui-lo, note que  você pode fazer o que desejar com ele. Se um
 * dia a gente se conhecer, e você achar que isso funciona, você pode nos comprar
 * uma cerveja em homenagem a Juliana Camilo Inácio e Thiago Martins de Sousa
 * ----------------------------------------------------------------------------
 */


package pintorj2d;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import javax.swing.JPanel;
import javax.swing.Timer;

/**
 *
 * Classe Area é filha da classe jPanel e implementa a interface ActionListener
 * essa classe Area mostra na tela as formas desenhadas, o fundo branco e trata
 * os eventos do mouse que ocorrem nessa area
 */
public class Area extends JPanel implements ActionListener {

    private Timer timer; // temporização utilizada para repintar a tela
    private int origemx; // armazena coordenadas no instante do clic
    private int origemy; // armazena coordenadas no instante do clic
    private int destinox; // valor da coordenada no momento em que se mexe o mouse ou que se solta o mouse
    private int destinoy; // valor da coordenada no momento em que se mexe o mouse ou que se solta o mouse
    private JanelaPrincipal pai; // variavel do tipo JanelaPrincipal.É a parte da tela que contem as ferramentas de desenho
    public ArrayList<FormaGeometrica> formasDesenhadas; // É um array que armazena todas as formas desenhadas ate o momento

    /**
     * Construtor recebendo o objeto do tipo janelaPrincipal
     *
     * @param jp
     */
    public Area(JanelaPrincipal jp) {
        this.pai = jp;// Recebe uma janela principal e armazena na variavel pai
        // onde iremos armazenar as formas desenhadas na área de desenho
        this.formasDesenhadas = new ArrayList<FormaGeometrica>();// intanciando o array


        // adicionando tratadores de eventos do mouse
        this.addMouseListener(new TratarEventosBotaoMouse());
        this.addMouseMotionListener(new TratarEventosMovimentoMouse());

        // alterando algumas propriedades do objeto
        this.setFocusable(true); // pode assumir foco
        this.setBackground(Color.WHITE); // alterando a cor do fundo
        this.setDoubleBuffered(true); // tornando o redesenho mais rapido

        /* Irá atualizar a tela a cada 100 milisegundos */
        this.timer = new Timer(100, this);
        this.timer.start();

        this.zerarCoordenadas();// seta as coordenadas para -1,-1
    }

    /**
     * Método que manda as formas geometricas desenharem
     * e é executado quando o repaint é chamado
     *
     * @param g
     */
    @Override
    public void paint(Graphics g) {
        super.paint(g);
        // cast para converter grafico para grafico 2D
        Graphics2D g2d = (Graphics2D) g;

        // ativando o antialiasing
        g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

// if para evitar erros na leitura do arquivo texto ao clicar na Area
        if ((pai.f != null) || (formasDesenhadas.size() > 1)) {
            // imprimindo na tela as formas que já foram desenhadas
            this.desenharFormasAnteriores(g2d);
        }
        if (pai.operacao == 1) {
            // compara se a forma selecionada é uma Linha pois esta receberá tratamento diferenciado devido as coordenadas
            if (pai.f instanceof Linha) {
                pai.f.setX(origemx);
                pai.f.setY(origemy);
                pai.f.setlargura(destinox);//É tratado como o ponto x2  da linha
                pai.f.setAltura(destinoy);//É tratado como o ponto y2  da linha
                pai.f.desenhar(g2d);

            } else {

                if (destinox != -1) {
                    int x, la, y, al;
                    // para descobrir se o x1 é menor ou maior que x2 para entao
                    // calcular a largura
                    if (origemx < destinox) {
                        x = origemx;
                        la = destinox - origemx;
                    } else {
                        x = destinox;
                        la = origemx - destinox;
                    }
                    /*
                     *O if acima serve para definir o ponto de destino e a largura
                     * Se o desenho ter inicio no lado esquerdo e fim no lado  direito
                     *entra no if, senão entra no else
                     */

                    // para descobrir se o y1 é menor ou maior que y2 para entao  calcular a altura
                    if (origemy < destinoy) {
                        y = origemy;
                        al = destinoy - origemy;
                    } else {
                        y = destinoy;
                        al = origemy - destinoy;
                    }
                    //Altera os parametros das formas e por ultimo desenha
                    if (pai.f != null) {
                        pai.f.setX(x);
                        pai.f.setY(y);
                        pai.f.setAltura(al);
                        pai.f.setlargura(la);
                        pai.f.desenhar(g2d);
                    }
                }
            }
        }
    }

    /**
     * Faz com que as coordenadas de clique sejam zeradas com valores não possíveis
     * na área de desenho.
     */
    public void zerarCoordenadas() {
        origemx = -1;
        origemy = -1;
        destinox = -1;
        destinoy = -1;
    }

    /**
     * Será executado cada N milisegundos. Ou seja, a tela será redesenhada.
     * Esse metodo é executado que ha um evento na Area
     * A atualização constante da tela é necessária para permitir o efeito
     * instantâneo de clicar e arrastar para desenhar uma forma
     *
     * @param e
     */
    public void actionPerformed(ActionEvent e) {
        // pintando a tela novamente
        repaint();
    }

    /**
     * Este metodo irá percorrer todas as formas já desenhadas e armazenadas no
     * ArrayList para desenhá-las na tela
     *
     * @param g2d - Tela onde será desenhada as imagens
     */
    private void desenharFormasAnteriores(Graphics2D g2d) {

        if (formasDesenhadas != null) {
            for (FormaGeometrica forma : formasDesenhadas) {
                forma.desenhar(g2d);
            }
        }
    }

    /**
     * Esse metodo apaga o array de FormasGeometricas e limpando a tela
     */
    public void limparImagem() {
        this.formasDesenhadas.clear();
        this.zerarCoordenadas();
    }

    /**
     * Este metodo retorna um vetor de formas geometricas
     *
     * @return FormaGeometrica[]
     */
    public FormaGeometrica[] retornaFormas() {
        int n = formasDesenhadas.size();
        FormaGeometrica[] forma = new FormaGeometrica[n];
        int k = 0;
        // Aqui o vetor formasDesenhadas é copiado para o vetor forma
        for (FormaGeometrica formaAux : formasDesenhadas) {
            forma[k] = formaAux;
            k++;
        }
        return forma;
    }

    /**
     * Este metodo apagará a última forma que tenha intersecção com a posiçao x e com a
     * posiçao y
     *
     * @param posx
     * @param posy
     */
    public void apagarImagem(int posx, int posy) {
        //O vetor será varrido de trás para frente, de modo que ira apagar a última forma
        // que foi desenhada e que compartilhe uma intersecção com uma ou mais formas.
        //o -1 é para a posição zero também ser varrida

        Object forma[] = formasDesenhadas.toArray();
        for (int i = forma.length - 1; i >= 0; i--) {
            if (((FormaGeometrica) forma[i]).interseccao(posx, posy)) {
                formasDesenhadas.remove(((FormaGeometrica) forma[i]));
                break;
            }
        }
    }

    /**
     * Este metodo colorir a última forma na intersecçao da posiçao x e posiçao y com
     * a cor selecionda na janelaPrincipal
     *
     * @param posx
     * @param posy
     */
    public void redesenharColorindo(int posx, int posy) {
        // O vetor será varrido de trás para frente, de modo que ira apagar a última forma
        // que foi desenhada e que compartilhe uma intersecção com uma ou mais formas.
        //o -1 é para a posição zero também ser varrida

        Object forma[] = formasDesenhadas.toArray();
        for (int i = forma.length - 1; i >= 0; i--) {
            if (((FormaGeometrica) forma[i]).interseccao(posx, posy)) {
                ((FormaGeometrica) forma[i]).setCorDeFundo(pai.corFundo);
                break;
            }
        }
    }

    /**
     * Classe interna e privada para tratar eventos de botão do mouse
     *
     * Uma classe interna consegue acessar todos os atributos (mesmo os privados)
     * da classe mais externa que a contém.
     *
     */
    private class TratarEventosBotaoMouse extends MouseAdapter {

        /**
         * Trata eventos de quando o mouse é clicado
         *
         * @param e
         */
        @Override
        public void mouseClicked(MouseEvent e) {
            if (pai.operacao == 3) {
                timer.setDelay(5);
                apagarImagem(e.getX(), e.getY());
            }
        }

        /**
         *  Trata os eventos quando o botão do mouse for pressionado
         * @param e evento do mouse
         */
        @Override
        public void mousePressed(MouseEvent e) {

            // Quando o mouse é pressionado ele salva as coordenadas de origem em origemx e origemy


            // Apos clicar na tela pra preenchimento da forma ele retorna a cor original do jP que esta armazenda em aux

            if ((pai.operacao == 1) && (pai.f != null)) {
                if (pai.f instanceof Linha) {
                    origemx = e.getX();
                    origemy = e.getY();
                    destinox = e.getX();
                    destinoy = e.getY();
                    pai.f.setX(origemx);
                    pai.f.setY(origemy);
                    pai.f.setAltura(origemx);
                    pai.f.setlargura(origemy);
                } else {
                    destinox = -1;
                    destinoy = -1;
                    origemx = e.getX();
                    origemy = e.getY();
                    pai.f.setX(origemx);
                    pai.f.setY(origemy);
                }
                // Quando se está desenhando algo, então a tela será atualizada a
                // cada 5 milisegundos
                // acessando atributo timer da classe externa
                timer.setDelay(5);
            } else if (pai.operacao == 2) {
                timer.setDelay(5);
                redesenharColorindo(e.getX(), e.getY());
                // Operação de apagar uma forma da area de desenho
            }
        }

        /**
         * Trata os eventos quando o botão do mouse for liberado
         *
         * @param e evento do mouse
         */
        @Override
        public void mouseReleased(MouseEvent e) {

            // Quando o mouse parar de se mexer e for liberado salva as coordenadas destinos
            //em destinox e destinoy

            if (pai.f != null) {
                if (pai.operacao == 1) {
                    // Quando não se está desenhando algo, então a tela será atualizada a
                    // cada 100 milisegundos. Isso faz com que nosso editor não fique
                    // consumindo muito o CPU.
                    // acessando atributo timer da classe externa
                    timer.setDelay(100);
                    destinox = e.getX();
                    destinoy = e.getY();
                    if (pai.f instanceof Linha) {
                        pai.f.setlargura(destinox);
                        pai.f.setAltura(destinoy);
                        pai.f.corDaLinha = pai.corLinha;//seta a cor ultimo item do codigo a ser alterado
                    } else {
                        int x, la, y, al;
                        // para descobrir se o x1 é menor ou maior que x2 para entao
                        // calcular a largura
                        if (origemx < destinox) {
                            x = origemx;
                            la = destinox - origemx;
                        } else {
                            x = destinox;
                            la = origemx - destinox;
                        }
                        // para descobrir se o y1 é menor ou maior que y2 para entao
                        // calcular a altura
                        if (origemy < destinoy) {
                            y = origemy;
                            al = destinoy - origemy;
                        } else {
                            y = destinoy;
                            al = origemy - destinoy;
                        }
                        if (pai.f != null) {
                            pai.f.corDaLinha = pai.corLinha;
                            pai.f.setAltura(al);
                            pai.f.setlargura(la);
                        }
                    }
                    // guardando a forma desenhada
                    // acessando atributo formasDesenhadas da classe externa
                    // e acessando o atributo formGeometrica da classe JanelaPrincipal
                    formasDesenhadas.add(pai.f);
                    if (pai.f instanceof Retangulo) {
                        pai.f = new Retangulo(-1, -1, 0, 0, pai.corLinha);
                    } else if (pai.f instanceof Elipse) {
                        pai.f = new Elipse(-1, -1, 0, 0, pai.corLinha);
                    } else if (pai.f instanceof Linha) {
                        pai.f = new Linha(origemx, origemy, origemx, origemy, pai.corLinha);
                    } else if (pai.f instanceof RetanguloArredondado) {
                        pai.f = new RetanguloArredondado(-1, -1, 0, 0, pai.corLinha);
                    }

                }
            }
        }
    }

    /**
     * Classe interna e privada para tratar eventos de movimento do mouse.
     * Uma classe interna consegue acessar todos os atributos (mesmo os privados)
     * da classe mais externa que a contém.
     */
    private class TratarEventosMovimentoMouse extends MouseAdapter {

        /**
         * Trata os eventos quando o mouse for movimentado sem pressionar o botão
         * @param e evento do mouse
         */
        @Override
        public void mouseMoved(MouseEvent e) {

            
            // Essa classe monitora todos os movimentos do mouse e salva a posição
             // do mouse no jLabel1 e JLabel2 (que são mostrados na tela tela)
             
            // atualizando os labels que representam as coordenadas na interface
            // do usuário
            // acessando os atributos jLabel1 e jLabel2 da classe JanelaPrincipal
            pai.jLabel1.setText("" + e.getX());
            pai.jLabel2.setText("" + e.getY());
        }

        /**
         * Trata os eventos quando o mouse for movimentado pressionando o botão
         * @param e evento do mouse
         */
        @Override
        public void mouseDragged(MouseEvent e) {
            if (pai.operacao == 1) {
                destinox = e.getX();
                destinoy = e.getY();
                if (pai.f != null) {
                    pai.f.corDaLinha = pai.corLinha;
                }
                
                // Essa classe monitora todos os movimentos do mouse quando ele é
                 // arrastado e salva a posição do mouse na variável
                 //destinox e destino y
                 //atualizando os labels que representam as coordenadas na interface  do usuário
                // acessando os atributos jLabel1 e jLabel2 da classe JanelaPrincipal
                pai.jLabel1.setText("" + e.getX());
                pai.jLabel2.setText("" + e.getY());
            }
        }
    }
}
        
    


    


